require 'urbanairship'

module Nuge
  module Clients
    class Urbanairship
      attr_reader :options
      attr_writer :expander

      def initialize(options = {})
        @expander = options[:expander]
        @options  = options
      end

      def pusher
        @pusher ||= ::Urbanairship.tap do |serv|
          serv.application_key    = options[:application_key]
          serv.application_secret = options[:application_secret]
          serv.master_secret      = options[:master_secret]
        end
      end

      def register(id, options = {})
        pusher.register_device(id, options)
      end

      def unregister(id, options = {})
        pusher.unregister_device(id, options)
      end

      def push(ids, message)
        formatter = message.delete(:formatter) || Nuge::Formatters::Aps

        pusher.push(formatter.call(ids, message))
      end
    end
  end

  module Formatters
    module Aps
      def self.call(ids, message)
        { device_tokens: ids, aps: message }
      end
    end
  end
end
