require 'set'

module Nuge
  class Pusher
    def self.clients=(clients)
      @clients = Set.new(clients)
    end

    def self.clients
      @clients ||= Set.new
    end

    attr_reader :clients
    attr_writer :expander

    def initialize(clients = nil)
      @clients = Set.new(clients || self.class.clients)
    end

    def push(ids, message)
      clients.each { |client| client.push(expanded(ids), message) }
    end

    def register(id, options = {})
      clients.each { |client| client.register(expanded(id).first, options) }
    end

    def unregister(id, options = {})
      clients.each { |client| client.unregister(expanded(id).first, options) }
    end

    def expander
      @expander ||= -> (obj) { obj }
    end

    def expanded(*objects)
      objects.flatten.map { |obj| expander.call(obj) }
    end
  end
end
